/*
 * macros.c --
 *
 * Defines and retrieves macros
 *
 *     ********************************************************************* 
 *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
 *     * Permission to use, copy, modify, and distribute this              * 
 *     * software and its documentation for any purpose and without        * 
 *     * fee is hereby granted, provided that the above copyright          * 
 *     * notice appear in all copies.  The University of California        * 
 *     * makes no representations about the suitability of this            * 
 *     * software for any purpose.  It is provided "as is" without         * 
 *     * express or implied warranty.  Export of this software outside     * 
 *     * of the United States of America may require an export license.    * 
 *     *********************************************************************
 */

#ifndef lint
static char rcsid[]="$Header: macros.c,v 6.0 90/08/28 18:47:03 mayo Exp $";
#endif  not lint

#include <stdio.h>
#include <signal.h>
#ifdef XLIB
#include <X11/X.h>
#include <X11/Xlib.h>
#endif
#include "magic.h"
#include "utils.h"
#include "macros.h"



/* 
 * Local data structures
 *
 */

static char *(macTab[128]), *(macTabX[2048]);
static char macInt[128], macIntX[2048];



/*---------------------------------------------------------
 * MacroDefine:
 *	This procedure defines a macro.
 *
 * Results:	None.
 *
 * Side Effects:
 *	The string passed is copied and considered to be the
 *	macro definition for the character.
 *---------------------------------------------------------
 */

void
MacroDefine(c, xc, str)
char c;			/* The single character name of the macro */
int xc;			/* the extended name of the macro */
char *str;		/* ...and the string to be attached to it */
{
    c &= 0x7F;
    xc &= 0x0FFF;
    if (xc != 0x0000)
    {
	(void) StrDup(&(macTabX[xc]), str);
	macIntX[xc] = '\0';
    }
    else
    {
	(void) StrDup(&(macTab[c]), str);
	macInt[c] = '\0';
    }
}


/*---------------------------------------------------------
 * MacroDefineInt:
 *	This procedure defines a macro.
 *
 * Results:	None.
 *
 * Side Effects:
 *	The string passed is copied and considered to be the
 *	macro definition for the character.
 *---------------------------------------------------------
 */

void
MacroDefineInt(c, xc, str)
char c;			/* The single character name of the macro */
int xc;			/* the extended name of the macro */
char *str;		/* ...and the string to be attached to it */
{
    c &= 0x7F;
    xc &= 0x0FFF;
    if (xc != 0x0000)
    {
	(void) StrDup(&(macTabX[xc]), str);
	macIntX[xc] = 'i';
    }
    else
    {
	(void) StrDup(&(macTab[c]), str);
	macInt[c] = 'i';
    }
}


/*---------------------------------------------------------
 * MacroRetrieve:
 *	This procedure retrieves a macro.
 *
 * Results:
 *	A pointer to a new Malloc'ed string is returned.
 *	This structure should be freed when the caller is
 *	done with it.
 *
 * Side Effects:
 *	None.
 *---------------------------------------------------------
 */

char *
MacroRetrieve(c, xc, iReturn)
char c;			/* the single character name of the macro */
int xc;			/* the extended name of the macro */
int *iReturn;
{
    c &= 0x7F;
    xc &= 0x0FFF;
    if (xc != 0x0000)
    {
	if (iReturn != NULL) {
	    if (macIntX[xc] == '\0') (*iReturn) = 0;
	    else (*iReturn) = 1;
	}
	if (macTabX[xc] != NULL)
	    return ( StrDup( (char **) NULL, macTabX[xc]) );
    }
    if (iReturn != NULL) {
      if (macInt[c] == '\0') (*iReturn) = 0;
      else (*iReturn) = 1;
    }
    return ( StrDup( (char **) NULL, macTab[c]) );
}


/*---------------------------------------------------------
 * MacroDelete:
 *	This procedure deletes a macro.
 *
 * Results:	None.
 *
 * Side Effects:
 *	The string that defines a macro is deleted.  This means
 *	that if anybody still has a pointer to that string
 *	they are in trouble.
 *---------------------------------------------------------
 */

void
MacroDelete(c, xc)
char c;			/* the single character name of the macro */
int xc;			/* the extended name of the macro */
{
    c &= 0x7F;
    xc &= 0x0FFF;
    if (xc != 0x0000)
    {
	(void) StrDup(&(macTabX[xc]), (char *) NULL);
	macIntX[xc] = '\0';
    }
    else
    {
	(void) StrDup(&(macTab[c]), (char *) NULL);
	macInt[c] = '\0';
    }
}



/*
 * ----------------------------------------------------------------------------
 * MacroName:
 * 	Convert an extended keycode to a string.
 *
 * Results:
 *	The string.
 *
 * Side effects:
 *	The string is malloc'ed from memory, and must be free'd later.
 * ----------------------------------------------------------------------------
 */

char *
MacroName(c, xc)
char c;
int xc;
{
    char *vis;
    static char hex[17] = "0123456789ABCDEF";
#ifdef XLIB
    char *str;
    extern Display *grXdpy;
    KeySym ks;
#endif

    xc &= 0x0FFF;
    if (xc != 0x0000)
    {
#ifdef XLIB
	str = NULL;
	if (grXdpy != NULL) {
	    ks = XKeycodeToKeysym (grXdpy, (KeyCode) (xc&0x00FF), 0);
	    if (ks != NoSymbol) str = XKeysymToString(ks);
	}
	if (str != NULL)
	{
	    vis = (char *) mallocMagic( sizeof(char) * (strlen(str) + 32) );
	    vis[0] = '\0';
	    if (xc & 0x0800) strcat(vis, "Meta_");
	    if (xc & 0x0400) strcat(vis, "Control_");
	    if (xc & 0x0200) strcat(vis, "Capslock_");
	    if (xc & 0x0100) strcat(vis, "Shift_");
	    strcat(vis, "XK_");
	    strcat(vis, str);
	    
	    return(vis);
	}
#endif
	vis = (char *) mallocMagic( sizeof(char) * 7 );
	vis[0] = '0';
	vis[1] = 'x';
	vis[2] = hex[ (xc&0xF000) >> 12];
	vis[3] = hex[ (xc&0x0F00) >> 8];
	vis[4] = hex[ (xc&0x00F0) >> 4];
	vis[5] = hex[ (xc&0x000F) ];
	vis[6] = '\0';

	return(vis);
    }

    vis = (char *) mallocMagic( sizeof(char) * 6 );
    if (c < ' ')
    {
	vis[0] = '^';
	vis[1] = c + '@';
	vis[2] = '\0';
    }
    else if (c == 0x7F)
    {
	vis[0] = '<';
	vis[1] = 'd';
	vis[2] = 'e';
	vis[3] = 'l';
	vis[4] = '>';
	vis[5] = '\0';
    }
    else
    {
	vis[0] = c;
	vis[1] = '\0';
    }
    return(vis);
}


/*
 * ----------------------------------------------------------------------------
 * MacroKey:
 * 	Convert a string to an extended keycode.
 *
 * Results:
 *	An extended macro name.
 *
 * Side effects:
 *	If the display being used is not X11, a warning is printed the
 *	first time this function is called.
 * ----------------------------------------------------------------------------
 */

char
MacroKey(str, kcp)
char *str;
int *kcp;
{
    static int warn = 1;

#ifdef XLIB
    int kc;
    char *vis;
    extern Display *grXdpy;
    KeySym ks;

    if ((grXdpy != NULL) && (strlen(str) != 1))
    {
	kc = 0x0000;
	vis = str;
	while( (*vis) != '\0' )
	{
	    if (!strncmp(vis, "Meta_", 5))
	    {
		kc|=0x0800;
		vis += 5;
	    }
	    else if (!strncmp(vis, "Control_", 8))
	    {
		kc|=0x0400;
		vis += 8;
	    }
	    else if (!strncmp(vis, "Capslock_", 9))
	    {
		kc|=0x0200;
		vis += 9;
	    }
	    else if (!strncmp(vis, "Shift_", 6))
	    {
		kc|=0x0100;
		vis += 6;
	    }
	    else break;
	}
	if (!strncmp(vis, "XK_", 3)) vis += 3;
	ks = XStringToKeysym(vis);
	if (ks != NoSymbol) kc |= (XKeysymToKeycode(grXdpy, ks) & 0x00FF);
	else kc = 0;
	(*kcp) = kc;
	return('\0');
    }
#endif
    if (strlen(str) == 1)
    {
	(*kcp) = 0;
	return(str[0]);
    }
    if (warn)
	TxError("Extended macros are unavailable with this device type.\n");
    warn = 0;
    (*kcp) = 0;
    return('\0');
}

